@@ -1,4 +1,4 @@
-/*	$Id: latex.c,v 1.24 2021/02/16 23:26:35 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2020--2021 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -33,7 +33,7 @@
 
 struct latex {
 	unsigned int	oflags; /* same as in lowdown_opts */
-	size_t		base_header_level; /* header offset */
+	ssize_t		headers_offs; /* header offset */
 };
 
 /*
@@ -267,13 +267,16 @@ rndr_header(struct lowdown_buf *ob,
 	const struct latex *st)
 {
 	const char	*type;
+	ssize_t		 level;
 
 	if (ob->size && !HBUF_PUTSL(ob, "\n"))
 		return 0;
 
-	switch (param->level + st->base_header_level) {
-	case 0:
-		/* FALLTHROUGH */
+	level = (ssize_t)param->level + st->headers_offs;
+	if (level < 1)
+		level = 1;
+
+	switch (level) {
 	case 1:
 		type = "\\section";
 		break;
@@ -354,9 +357,18 @@ rndr_listitem(struct lowdown_buf *ob,
 
 	/* Only emit <li> if we're not a <dl> list. */
 
-	if (!(param->flags & HLIST_FL_DEF) &&
-	    !HBUF_PUTSL(ob, "\\item "))
-		return 0;
+	if (!(param->flags & HLIST_FL_DEF)) {
+		if (!HBUF_PUTSL(ob, "\\item"))
+			return 0;
+		if ((param->flags & HLIST_FL_CHECKED) &&
+		    !HBUF_PUTSL(ob, "[$\\rlap{$\\checkmark$}\\square$]"))
+			return 0;
+		if ((param->flags & HLIST_FL_UNCHECKED) &&
+		    !HBUF_PUTSL(ob, "[$\\square$]"))
+			return 0;
+		if (!HBUF_PUTSL(ob, " "))
+			return 0;
+	}
 
 	/* Cut off any trailing space. */
 
@@ -597,7 +609,7 @@ rndr_footnote_def(struct lowdown_buf *ob,
 	const struct rndr_footnote_def *param)
 {
 
-	if (!hbuf_printf(ob, "\\footnotetext[%zu]{", param->num + 1))
+	if (!hbuf_printf(ob, "\\footnotetext[%zu]{", param->num))
 		return 0;
 	if (n->chng == LOWDOWN_CHNG_INSERT &&
 	    !HBUF_PUTSL(ob, "\\textcolor{blue}{"))
@@ -619,7 +631,7 @@ rndr_footnote_ref(struct lowdown_buf *ob,
 	const struct rndr_footnote_ref *param)
 {
 
-	return hbuf_printf(ob, "\\footnotemark[%zu]", param->num + 1);
+	return hbuf_printf(ob, "\\footnotemark[%zu]", param->num);
 }
 
 static int
@@ -671,6 +683,8 @@ rndr_doc_header(struct lowdown_buf *ob,
 	    "\\usepackage{textcomp}\n"
 	    "\\usepackage{lmodern}\n"
 	    "\\usepackage{hyperref}\n"
+	    "\\usepackage{amsmath}\n"
+	    "\\usepackage{amssymb}\n"
 	    "\\begin{document}\n"))
 		return 0;
 
@@ -723,6 +737,8 @@ rndr_meta(struct lowdown_buf *ob,
 	const struct lowdown_node *n, struct latex *st)
 {
 	struct lowdown_meta	*m;
+	ssize_t			 val;
+	const char		*ep;
 
 	if ((m = calloc(1, sizeof(struct lowdown_meta))) == NULL)
 		return 0;
@@ -736,11 +752,16 @@ rndr_meta(struct lowdown_buf *ob,
 	if (m->value == NULL)
 		return 0;
 
-	if (strcasecmp(m->key, "baseheaderlevel") == 0) {
-		st->base_header_level = strtonum
-			(m->value, 1, 1000, NULL);
-		if (st->base_header_level == 0)
-			st->base_header_level = 1;
+	if (strcmp(m->key, "shiftheadinglevelby") == 0) {
+		val = (ssize_t)strtonum
+			(m->value, -100, 100, &ep);
+		if (ep == NULL)
+			st->headers_offs = val + 1;
+	} else if (strcmp(m->key, "baseheaderlevel") == 0) {
+		val = (ssize_t)strtonum
+			(m->value, 1, 100, &ep);
+		if (ep == NULL)
+			st->headers_offs = val;
 	}
 
 	return 1;
@@ -897,28 +918,18 @@ out:
 
 int
 lowdown_latex_rndr(struct lowdown_buf *ob,
-	struct lowdown_metaq *mq, void *arg, 
-	const struct lowdown_node *n)
+	void *arg, const struct lowdown_node *n)
 {
 	struct latex		*st = arg;
 	struct lowdown_metaq	 metaq;
 	int			 rc;
 
-	/* Temporary metaq if not provided. */
+	TAILQ_INIT(&metaq);
+	st->headers_offs = 1;
 
-	if (mq == NULL) {
-		TAILQ_INIT(&metaq);
-		mq = &metaq;
-	}
-
-	st->base_header_level = 1;
-	rc = rndr(ob, mq, st, n);
-
-	/* Release temporary metaq. */
-
-	if (mq == &metaq)
-		lowdown_metaq_free(mq);
+	rc = rndr(ob, &metaq, st, n);
 
+	lowdown_metaq_free(&metaq);
 	return rc;
 }
 
