@@ -1,4 +1,4 @@
-/*	$Id: nroff.c,v 1.150 2021/02/17 09:42:21 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2008, Natacha Porté
  * Copyright (c) 2011, Vicent Martí
@@ -44,13 +44,13 @@ struct 	nroff {
 	int 		 man; /* whether man(7) */
 	int		 post_para; /* for choosing PP/LP */
 	unsigned int 	 flags; /* output flags */
-	size_t		 base_header_level; /* header offset */
+	ssize_t		 headers_offs; /* header offset */
 	enum nfont	 fonts[NFONT__MAX]; /* see bqueue_font() */
 };
 
 enum	bscope {
-	BSCOPE_BLOCK,
-	BSCOPE_SPAN,
+	BSCOPE_BLOCK = 0,
+	BSCOPE_SPAN = 1,
 	BSCOPE_LITERAL,
 	BSCOPE_FONT,
 	BSCOPE_COLOUR
@@ -65,7 +65,7 @@ enum	bscope {
  */
 struct	bnode {
 	char				*nbuf; /* (safe) 1st data */
-	const struct lowdown_buf	*buf; /* (unsafe) 2nd data */
+	char				*buf; /* (unsafe) 2nd data */
 	size_t				 bufchop; /* strip from buf */
 	char				*nargs; /* (safe) 1st args */
 	char				*args; /* (unsafe) 2nd args */
@@ -93,12 +93,14 @@ TAILQ_HEAD(bnodeq, bnode);
  */
 static int
 hesc_nroff(struct lowdown_buf *ob, const char *data, 
-	size_t size, int oneline, int literal)
+	size_t size, int oneline, int literal, int esc)
 {
 	size_t	 i = 0;
 
 	if (size == 0)
 		return 1;
+	if (!esc)
+		return hbuf_put(ob, data, size);
 
 	/* Strip leading whitespace. */
 
@@ -297,6 +299,7 @@ bnode_free(struct bnode *bn)
 	free(bn->args);
 	free(bn->nargs);
 	free(bn->nbuf);
+	free(bn->buf);
 	free(bn);
 }
 
@@ -329,7 +332,7 @@ bqueue_strip_paras(struct bnodeq *bq)
 }
 
 static int
-bqueue_flush(struct lowdown_buf *ob, const struct bnodeq *bq)
+bqueue_flush(struct lowdown_buf *ob, const struct bnodeq *bq, int esc)
 {
 	const struct bnode	*bn, *chk;
 	const char		*cp;
@@ -413,13 +416,12 @@ bqueue_flush(struct lowdown_buf *ob, const struct bnodeq *bq)
 
 		if (bn->scope == BSCOPE_LITERAL) {
 			assert(bn->buf != NULL);
-			if (!hesc_nroff(ob,
-			    bn->buf->data, bn->buf->size, 0, 1))
+			if (!hesc_nroff(ob, bn->buf,
+			    strlen(bn->buf), 0, 1, esc))
 				return 0;
 		} else if (bn->buf != NULL)
-			if (!hesc_nroff(ob,
-			    bn->buf->data + bn->bufchop, 
-			    bn->buf->size - bn->bufchop, 0, 0))
+			if (!hesc_nroff(ob, bn->buf + bn->bufchop, 
+			    strlen(bn->buf) - bn->bufchop, 0, 0, esc))
 				return 0;
 
 		/* 
@@ -443,8 +445,8 @@ bqueue_flush(struct lowdown_buf *ob, const struct bnodeq *bq)
 			assert(bn->scope == BSCOPE_BLOCK);
 			if (!hbuf_putc(ob, ' '))
 				return 0;
-			if (!hesc_nroff(ob,
-			    bn->args, strlen(bn->args), 1, 0))
+			if (!hesc_nroff(ob, bn->args,
+			    strlen(bn->args), 1, 0, esc))
 				return 0;
 		}
 
@@ -469,7 +471,7 @@ bqueue_to_nargs(struct bnode *bn, const struct bnodeq *bq, int quoted)
 		goto out;
 	if (quoted && !hbuf_putc(ob, '"'))
 		goto out;
-	if (!bqueue_flush(ob, bq))
+	if (!bqueue_flush(ob, bq, 1))
 		goto out;
 	if (quoted && !hbuf_putc(ob, '"'))
 		goto out;
@@ -499,8 +501,7 @@ hbuf2shortlink(const struct lowdown_buf *link)
 		goto out;
 	if (!hbuf_shortlink(tmp, link))
 		goto out;
-	if (!hesc_nroff(slink, 
-	    tmp->data, tmp->size, 1, 0))
+	if (!hesc_nroff(slink, tmp->data, tmp->size, 1, 0, 1))
 		goto out;
 	ret = strndup(slink->data, slink->size);
 out:
@@ -547,8 +548,11 @@ putlink(struct bnodeq *obq, struct nroff *st,
 				bn->nbuf = hbuf2shortlink(link);
 				if (bn->nbuf == NULL)
 					goto out;
-			} else
-				bn->buf = link;
+			} else {
+				bn->buf = strndup(link->data, link->size);
+				if (bn->buf == NULL)
+					goto out;
+			}
 			st->fonts[NFONT_ITALIC]--;
 			if (!bqueue_font(st, obq, 1))
 				goto out;
@@ -577,8 +581,11 @@ putlink(struct bnodeq *obq, struct nroff *st,
 			bn->nbuf = hbuf2shortlink(link);
 			if (bn->nbuf == NULL)
 				goto out;
-		} else
- 			bn->buf = link;
+		} else {
+ 			bn->buf = strndup(link->data, link->size);
+			if (bn->buf == NULL)
+				goto out;
+		}
 		st->fonts[NFONT_ITALIC]--;
 		if (!bqueue_font(st, obq, 1))
 			goto out;
@@ -602,30 +609,31 @@ putlink(struct bnodeq *obq, struct nroff *st,
 	if (prev != NULL &&
 	    prev->scope == BSCOPE_SPAN &&
 	    prev->buf != NULL &&
-	    prev->buf->size > 0 && !isspace
-	    ((unsigned char)prev->buf->data[prev->buf->size - 1])) {
-		sz = prev->buf->size;
+	    strlen(prev->buf) > 0 && !isspace
+	    ((unsigned char)prev->buf[strlen(prev->buf) - 1])) {
+		sz = strlen(prev->buf);
 		while (sz && !isspace
-		       ((unsigned char)prev->buf->data[sz - 1]))
+		       ((unsigned char)prev->buf[sz - 1]))
 			sz--;
-		assert(sz != prev->buf->size);
+		assert(sz != strlen(prev->buf));
 
 		if (!HBUF_PUTSL(ob, "-P \""))
 			goto out;
-		if (!hesc_nroff(ob, 
-		    &prev->buf->data[sz], prev->buf->size - sz, 1, 0))
+		if (!hesc_nroff(ob, &prev->buf[sz],
+		    strlen(prev->buf) - sz, 1, 0, 1))
 			goto out;
 		if (!HBUF_PUTSL(ob, "\" "))
 			goto out;
 
 		if ((tmp = hbuf_new(32)) == NULL)
 			goto out;
-		if (!hesc_nroff(tmp, prev->buf->data, sz, 1, 0))
+		if (!hesc_nroff(tmp, prev->buf, sz, 1, 0, 1))
 			goto out;
 		assert(prev->nbuf == NULL);
 		prev->nbuf = strndup(tmp->data, tmp->size);
 		if (prev->nbuf == NULL)
 			goto out;
+		free(prev->buf);
 		prev->buf = NULL;
 	}
 
@@ -644,7 +652,7 @@ putlink(struct bnodeq *obq, struct nroff *st,
 		if (sz > 0) {
 			if (!HBUF_PUTSL(ob, "-A \""))
 				goto out;
-			if (!hesc_nroff(ob, nbuf->data, sz, 1, 0))
+			if (!hesc_nroff(ob, nbuf->data, sz, 1, 0, 1))
 				goto out;
 			if (!HBUF_PUTSL(ob, "\" "))
 				goto out;
@@ -671,7 +679,7 @@ putlink(struct bnodeq *obq, struct nroff *st,
 		goto out;
 	if (bq == NULL && !hbuf_putb(ob, link))
 		goto out;
-	else if (bq != NULL && !bqueue_flush(ob, bq))
+	else if (bq != NULL && !bqueue_flush(ob, bq, 1))
 		goto out;
 	if ((bn = bqueue_block(obq, ".pdfhref W")) == NULL)
 		goto out;
@@ -726,7 +734,9 @@ rndr_blockcode(const struct nroff *st, struct bnodeq *obq,
 		return 0;
 	TAILQ_INSERT_TAIL(obq, bn, entries);
 	bn->scope = BSCOPE_LITERAL;
-	bn->buf = &param->text;
+	bn->buf = strndup(param->text.data, param->text.size);
+	if (bn->buf == NULL)
+		return 0;
 
 	if (st->man && (st->flags & LOWDOWN_NROFF_GROFF))
 		return bqueue_block(obq, ".EE") != NULL;
@@ -802,8 +812,8 @@ rndr_codespan(struct bnodeq *obq, const struct rndr_codespan *param)
 
 	if ((bn = bqueue_span(obq, NULL)) == NULL)
 		return 0;
-	bn->buf = &param->text;
-	return 1;
+	bn->buf = strndup(param->text.data, param->text.size);
+	return bn->buf != NULL;
 }
 
 static int
@@ -817,10 +827,12 @@ static int
 rndr_header(struct nroff *st, struct bnodeq *obq,
 	struct bnodeq *bq, const struct rndr_header *param)
 {
-	size_t			 level;
+	ssize_t			 level;
 	struct bnode		*bn;
 
-	level = param->level + st->base_header_level;
+	level = (ssize_t)param->level + st->headers_offs;
+	if (level < 1)
+		level = 1;
 
 	/*
 	 * For man(7), we use SH for the first-level section, SS for
@@ -851,7 +863,7 @@ rndr_header(struct nroff *st, struct bnodeq *obq,
 
 	if ((st->flags & LOWDOWN_NROFF_NUMBERED) ||
 	    (st->flags & LOWDOWN_NROFF_GROFF)) 
-		if (asprintf(&bn->nargs, "%zu", level) == -1) {
+		if (asprintf(&bn->nargs, "%zd", level) == -1) {
 			bn->nargs = NULL;
 			return 0;
 		}
@@ -887,6 +899,7 @@ rndr_listitem(struct bnodeq *obq, const struct lowdown_node *n,
 	struct bnodeq *bq, const struct rndr_listitem *param)
 {
 	struct bnode	*bn;
+	const char	*box;
 
 	if (param->flags & HLIST_FL_ORDERED) {
 		if ((bn = bqueue_block(obq, ".IP")) == NULL)
@@ -895,7 +908,15 @@ rndr_listitem(struct bnodeq *obq, const struct lowdown_node *n,
 		    "\"%zu.  \"", param->num) == -1)
 			return 0;
 	} else if (param->flags & HLIST_FL_UNORDERED) {
-		if (bqueue_block(obq, ".IP \"\\(bu\" 2") == NULL)
+		if (param->flags & HLIST_FL_CHECKED)
+			box = "[u2611]";
+		else if (param->flags & HLIST_FL_UNCHECKED)
+			box = "[u2610]";
+		else
+			box = "(bu";
+		if ((bn = bqueue_block(obq, ".IP")) == NULL)
+			return 0;
+		if (asprintf(&bn->nargs, "\"\\%s\" 2", box) == -1)
 			return 0;
 	}
 
@@ -969,8 +990,8 @@ rndr_raw_block(const struct nroff *st,
 		return 0;
 	TAILQ_INSERT_TAIL(obq, bn, entries);
 	bn->scope = BSCOPE_LITERAL;
-	bn->buf = &param->text;
-	return 1;
+	bn->buf = strndup(param->text.data, param->text.size);
+	return bn->buf != NULL;
 }
 
 static int
@@ -1005,7 +1026,9 @@ rndr_image(struct nroff *st, struct bnodeq *obq,
 			return 0;
 		if ((bn = bqueue_span(obq, NULL)) == NULL)
 			return 0;
-		bn->buf = &param->alt;
+		bn->buf = strndup(param->alt.data, param->alt.size);
+		if (bn->buf == NULL)
+			return 0;
 		st->fonts[NFONT_BOLD]--;
 		if (!bqueue_font(st, obq, 1))
 			return 0;
@@ -1022,8 +1045,11 @@ rndr_image(struct nroff *st, struct bnodeq *obq,
 			bn->nbuf = hbuf2shortlink(&param->link);
 			if (bn->nbuf == NULL)
 				return 0;
-		} else
-			bn->buf = &param->link;
+		} else {
+			bn->buf = strndup(param->link.data, param->link.size);
+			if (bn->buf == NULL)
+				return 0;
+		}
 		st->fonts[NFONT_ITALIC]--;
 		if (!bqueue_font(st, obq, 1))
 			return 0;
@@ -1063,15 +1089,18 @@ rndr_raw_html(const struct nroff *st,
 		return 0;
 	TAILQ_INSERT_TAIL(obq, bn, entries);
 	bn->scope = BSCOPE_LITERAL;
-	bn->buf = &param->text;
-	return 1;
+	bn->buf = strndup(param->text.data, param->text.size);
+	return bn->buf != NULL;
 }
 
 static int
 rndr_table(struct nroff *st, struct bnodeq *obq, struct bnodeq *bq)
 {
+	const char	*macro;
 
-	if (bqueue_block(obq, ".TS") == NULL)
+	macro = st->man || !(st->flags & LOWDOWN_NROFF_GROFF) ?
+		".TS" : ".TS H";
+	if (bqueue_block(obq, macro) == NULL)
 		return 0;
 	if (bqueue_block(obq, "tab(|) expand allbox;") == NULL)
 		return 0;
@@ -1081,8 +1110,8 @@ rndr_table(struct nroff *st, struct bnodeq *obq, struct bnodeq *bq)
 }
 
 static int
-rndr_table_header(struct bnodeq *obq, struct bnodeq *bq, 
-	const struct rndr_table_header *param)
+rndr_table_header(const struct nroff *st, struct bnodeq *obq,
+	struct bnodeq *bq, const struct rndr_table_header *param)
 {
 	size_t		 	 i;
 	struct lowdown_buf	*ob;
@@ -1149,6 +1178,11 @@ rndr_table_header(struct bnodeq *obq, struct bnodeq *bq,
 		goto out;
 
 	TAILQ_CONCAT(obq, bq, entries);
+
+	if (!st->man && (st->flags & LOWDOWN_NROFF_GROFF) &&
+	    bqueue_block(obq, ".TH") == NULL)
+		goto out;
+
 	rc = 1;
 out:
 	hbuf_free(ob);
@@ -1255,7 +1289,7 @@ rndr_footnote_def(const struct nroff *st, struct bnodeq *obq,
 	if ((bn = bqueue_span(obq, NULL)) == NULL)
 		return 0;
 	if (asprintf(&bn->nbuf, 
-	    "\\0\\fI\\u\\s-3%zu\\s+3\\d\\fP\\0", param->num + 1) == -1) {
+	    "\\0\\fI\\u\\s-3%zu\\s+3\\d\\fP\\0", param->num) == -1) {
 		bn->nbuf = NULL;
 		return 0;
 	}
@@ -1285,7 +1319,7 @@ rndr_footnote_ref(const struct nroff *st,
 
 	if (st->man) {
 		if (asprintf(&bn->nbuf, 
-		    "\\u\\s-3%zu\\s+3\\d", param->num + 1) == -1)
+		    "\\u\\s-3%zu\\s+3\\d", param->num) == -1)
 			bn->nbuf = NULL;
 	} else
 		bn->nbuf = strdup("\\**");
@@ -1312,8 +1346,8 @@ rndr_entity(const struct nroff *st,
 	} 
 	if ((bn = bqueue_span(obq, NULL)) == NULL)
 		return 0;
-	bn->buf = &param->text;
-	return 1;
+	bn->buf = strndup(param->text.data, param->text.size);
+	return bn->buf != NULL;
 }
 
 /*
@@ -1330,6 +1364,9 @@ rndr_meta_multi(struct bnodeq *obq, const char *b, const char *env)
 	struct bnode	*bn;
 	char		 macro[32];
 
+	if (b == NULL)
+		return 1;
+
 	assert(strlen(env) < sizeof(macro) - 1);
 	snprintf(macro, sizeof(macro), ".%s", env);
 	bsz = strlen(b);
@@ -1350,9 +1387,11 @@ rndr_meta_multi(struct bnodeq *obq, const char *b, const char *env)
 		if ((sz = &b[i] - start) == 0)
 			continue;
 
-		if ((bn = bqueue_block(obq, macro)) == NULL)
+		if (bqueue_block(obq, macro) == NULL)
 			return 0;
-		if ((bn->nargs = strndup(start, sz)) == NULL)
+		if ((bn = bqueue_span(obq, NULL)) == NULL)
+			return 0;
+		if ((bn->buf = strndup(start, sz)) == NULL)
 			return 0;
 	}
 
@@ -1365,6 +1404,8 @@ rndr_meta(struct nroff *st, const struct bnodeq *bq,
 {
 	struct lowdown_meta	*m;
 	struct lowdown_buf	*ob;
+	ssize_t			 val;
+	const char		*ep;
 
 	if ((m = calloc(1, sizeof(struct lowdown_meta))) == NULL)
 		return 0;
@@ -1376,7 +1417,7 @@ rndr_meta(struct nroff *st, const struct bnodeq *bq,
 
 	if ((ob = hbuf_new(32)) == NULL)
 		return 0;
-	if (!bqueue_flush(ob, bq)) {
+	if (!bqueue_flush(ob, bq, 0)) {
 		hbuf_free(ob);
 		return 0;
 	}
@@ -1385,12 +1426,18 @@ rndr_meta(struct nroff *st, const struct bnodeq *bq,
 	if (m->value == NULL)
 		return 0;
 
-	if (strcasecmp(m->key, "baseheaderlevel") == 0) {
-		st->base_header_level = strtonum
-			(m->value, 1, 1000, NULL);
-		if (st->base_header_level == 0)
-			st->base_header_level = 1;
+	if (strcmp(m->key, "shiftheadinglevelby") == 0) {
+		val = (ssize_t)strtonum
+			(m->value, -100, 100, &ep);
+		if (ep == NULL)
+			st->headers_offs = val + 1;
+	} else if (strcmp(m->key, "baseheaderlevel") == 0) {
+		val = (ssize_t)strtonum
+			(m->value, 1, 100, &ep);
+		if (ep == NULL)
+			st->headers_offs = val;
 	}
+
 	return 1;
 }
 
@@ -1404,7 +1451,7 @@ rndr_doc_header(const struct nroff *st,
 	int				 rc = 0;
 	const char			*author = NULL, *title = NULL,
 					*affil = NULL, *date = NULL,
-					*copy = NULL, *section = NULL,
+					*copy = NULL, *sec = NULL,
 					*rcsauthor = NULL, *rcsdate = NULL,
 					*source = NULL, *volume = NULL;
 
@@ -1427,7 +1474,7 @@ rndr_doc_header(const struct nroff *st,
 		else if (strcasecmp(m->key, "title") == 0)
 			title = m->value;
 		else if (strcasecmp(m->key, "section") == 0)
-			section = m->value;
+			sec = m->value;
 		else if (strcasecmp(m->key, "source") == 0)
 			source = m->value;
 		else if (strcasecmp(m->key, "volume") == 0)
@@ -1437,15 +1484,13 @@ rndr_doc_header(const struct nroff *st,
 
 	if (title == NULL)
 		title = "Untitled article";
-	if (section == NULL)
-		section = "7";
+	if (sec == NULL)
+		sec = "7";
 	if (rcsdate != NULL)
 		date = rcsdate;
 	if (rcsauthor != NULL)
 		author = rcsauthor;
 
-	/* Scratch space. */
-
 	bn = bqueue_block(obq, 
 		".\\\" -*- mode: troff; coding: utf-8 -*-");
 	if (bn == NULL)
@@ -1453,10 +1498,11 @@ rndr_doc_header(const struct nroff *st,
 
 	if (!st->man) {
 		if (copy != NULL) {
-			bn = bqueue_block(obq, ".ds LF Copyright \\(co");
+			bn = bqueue_block(obq,
+				".ds LF Copyright \\(co");
 			if (bn == NULL)
 				goto out;
-			if ((bn->nargs = strdup(copy)) == NULL)
+			if ((bn->args = strdup(copy)) == NULL)
 				goto out;
 		}
 		if (date != NULL) {
@@ -1466,18 +1512,18 @@ rndr_doc_header(const struct nroff *st,
 				bn = bqueue_block(obq, ".DA");
 			if (bn == NULL)
 				goto out;
-			if ((bn->nargs = strdup(date)) == NULL)
+			if ((bn->args = strdup(date)) == NULL)
 				goto out;
 		}
 		if (bqueue_block(obq, ".TL") == NULL)
 			goto out;
-		if (bqueue_block(obq, title) == NULL)
+		if ((bn = bqueue_span(obq, NULL)) == NULL)
 			goto out;
-		if (author != NULL && 
-		    !rndr_meta_multi(obq, author, "AU"))
+		if ((bn->buf = strdup(title)) == NULL)
 			goto out;
-		if (affil != NULL &&
-		    !rndr_meta_multi(obq, affil, "AI"))
+		if (!rndr_meta_multi(obq, author, "AU"))
+			goto out;
+		if (!rndr_meta_multi(obq, affil, "AI"))
 			goto out;
 	} else {
 		if ((ob = hbuf_new(32)) == NULL)
@@ -1491,9 +1537,9 @@ rndr_doc_header(const struct nroff *st,
 		if ((bn = bqueue_block(obq, ".TH")) == NULL)
 			goto out;
 		if (!hbuf_putc(ob, '"') ||
-		    !hbuf_puts(ob, title) ||
+		    !hesc_nroff(ob, title, strlen(title), 1, 0, 1) ||
 		    !HBUF_PUTSL(ob, "\" \"") ||
-		    !hbuf_puts(ob, section) ||
+		    !hesc_nroff(ob, sec, strlen(sec), 1, 0, 1) ||
 		    !hbuf_putc(ob, '"'))
 			goto out;
 
@@ -1502,15 +1548,13 @@ rndr_doc_header(const struct nroff *st,
 		 * case man(7) says the current date is used.
 		 */
 
-		if (date != NULL) {
-			if (!HBUF_PUTSL(ob, " \"") ||
-			    !hbuf_puts(ob, date) ||
-			    !HBUF_PUTSL(ob, "\""))
-				goto out;
-		} else {
-			if (!HBUF_PUTSL(ob, " \"\""))
-				goto out;
-		}
+		if (!HBUF_PUTSL(ob, " \""))
+			goto out;
+		if (date != NULL &&
+		    !hesc_nroff(ob, date, strlen(date), 1, 0, 1))
+			goto out;
+		if (!HBUF_PUTSL(ob, "\""))
+			goto out;
 
 		/*
 		 * Don't print these unless necessary, as the latter
@@ -1519,25 +1563,20 @@ rndr_doc_header(const struct nroff *st,
 		 */
 
 		if (source != NULL || volume != NULL) {
-			if (source != NULL) {
-				if (!HBUF_PUTSL(ob, " \"") ||
-				    !hbuf_puts(ob, source) ||
-				    !HBUF_PUTSL(ob, "\""))
-					goto out;
-			} else {
-				if (!HBUF_PUTSL(ob, " \"\""))
-					goto out;
-			}
-
-			if (volume != NULL) {
-				if (!HBUF_PUTSL(ob, " \"") ||
-				    !hbuf_puts(ob, volume) ||
-				    !HBUF_PUTSL(ob, "\""))
-					goto out;
-			} else if (source != NULL) {
-				if (!HBUF_PUTSL(ob, " \"\""))
-					goto out;
-			}
+			if (!HBUF_PUTSL(ob, " \""))
+				goto out;
+			if (source != NULL && !hesc_nroff
+			    (ob, source, strlen(source), 1, 0, 1))
+				goto out;
+			if (!HBUF_PUTSL(ob, "\""))
+				goto out;
+			if (!HBUF_PUTSL(ob, " \""))
+				goto out;
+			if (volume != NULL && !hesc_nroff
+			    (ob, volume, strlen(volume), 1, 0, 1))
+				goto out;
+			if (!HBUF_PUTSL(ob, "\""))
+				goto out;
 		}
 		if ((bn->nargs = strndup(ob->data, ob->size)) == NULL)
 			goto out;
@@ -1659,7 +1698,8 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 		rc = rndr_table(st, obq, &tmpbq);
 		break;
 	case LOWDOWN_TABLE_HEADER:
-		rc = rndr_table_header(obq, &tmpbq, &n->rndr_table_header);
+		rc = rndr_table_header(st, 
+			obq, &tmpbq, &n->rndr_table_header);
 		break;
 	case LOWDOWN_TABLE_ROW:
 		rc = rndr_table_row(obq, &tmpbq);
@@ -1708,7 +1748,11 @@ rndr(struct lowdown_metaq *mq, struct nroff *st,
 			break;
 		if ((bn = bqueue_span(obq, NULL)) == NULL)
 			goto out;
-		bn->buf = &n->rndr_normal_text.text;
+		bn->buf = strndup
+			(n->rndr_normal_text.text.data,
+			 n->rndr_normal_text.text.size);
+		if (bn->buf == NULL)
+			goto out;
 		bn->bufchop = chop;
 		break;
 	case LOWDOWN_ENTITY:
@@ -1757,28 +1801,22 @@ out:
 
 int
 lowdown_nroff_rndr(struct lowdown_buf *ob,
-	struct lowdown_metaq *mq, void *arg, 
-	const struct lowdown_node *n)
+	void *arg, const struct lowdown_node *n)
 {
 	struct nroff		*st = arg;
 	struct lowdown_metaq	 metaq;
 	int			 rc = 0;
 	struct bnodeq		 bq;
 
-	/* Temporary metaq if not provided. */
-
-	if (mq == NULL) {
-		TAILQ_INIT(&metaq);
-		mq = &metaq;
-	}
-
+	TAILQ_INIT(&metaq);
 	TAILQ_INIT(&bq);
+
 	memset(st->fonts, 0, sizeof(st->fonts));
-	st->base_header_level = 1;
+	st->headers_offs = 1;
 	st->post_para = 0;
 
-	if (rndr(mq, st, n, &bq, 0) >= 0) {
-		if (!bqueue_flush(ob, &bq))
+	if (rndr(&metaq, st, n, &bq, 0) >= 0) {
+		if (!bqueue_flush(ob, &bq, 1))
 			goto out;
 		if (ob->size && ob->data[ob->size - 1] != '\n' &&
 		    !hbuf_putc(ob, '\n'))
@@ -1786,8 +1824,7 @@ lowdown_nroff_rndr(struct lowdown_buf *ob,
 		rc = 1;
 	}
 out:
-	if (mq == &metaq)
-		lowdown_metaq_free(mq);
+	lowdown_metaq_free(&metaq);
 	bqueue_free(&bq);
 	return rc;
 }
