@@ -56,37 +56,31 @@ struct term {
  * How to style the output on the screen.
  */
 struct sty {
-	int	 italic;
-	int	 strike;
-	int	 bold;
-	int	 under;
-	size_t	 bcolour; /* not inherited */
-	size_t	 colour; /* not inherited */
-	int	 override; /* don't inherit */
-#define	OSTY_ITALIC	0x01
-#define	OSTY_BOLD	0x02
+	int		 italic; /* italic */
+	int		 strike; /* strikethrough */
+	int		 bold; /* bold */
+	int		 under; /* underline */
+	size_t		 bcolour; /* not inherited */
+	size_t		 colour; /* not inherited */
+	int		 override; /* don't inherit... */
+#define	OSTY_UNDER	 0x01 /* underlining */
+#define	OSTY_BOLD	 0x02 /* bold */
 };
 
-/* Per-node styles. */
+/*
+ * Prefixes to put before each line.  This only applies to very specific
+ * circumstances.
+ */
+struct	pfx {
+	const char	*text;
+	size_t		 cols;
+};
 
-static const struct sty sty_image =	{ 0, 0, 1, 0,   0, 92, 1 };
-static const struct sty sty_foot_ref =	{ 0, 0, 1, 0,   0, 92, 1 };
-static const struct sty sty_codespan = 	{ 0, 0, 0, 0,  47, 31, 0 };
-static const struct sty sty_hrule = 	{ 0, 0, 0, 0,   0, 37, 0 };
-static const struct sty sty_blockhtml =	{ 0, 0, 0, 0,   0, 37, 0 };
-static const struct sty sty_rawhtml = 	{ 0, 0, 0, 0,   0, 37, 0 };
-static const struct sty sty_strike = 	{ 0, 1, 0, 0,   0,  0, 0 };
-static const struct sty sty_emph = 	{ 1, 0, 0, 0,   0,  0, 0 };
-static const struct sty sty_highlight =	{ 0, 0, 1, 0,   0,  0, 0 };
-static const struct sty sty_d_emph = 	{ 0, 0, 1, 0,   0,  0, 0 };
-static const struct sty sty_t_emph = 	{ 1, 0, 1, 0,   0,  0, 0 };
-static const struct sty sty_link = 	{ 0, 0, 0, 1,   0, 32, 0 };
-static const struct sty sty_autolink =	{ 0, 0, 0, 1,   0, 32, 0 };
-static const struct sty sty_header =	{ 0, 0, 1, 0,   0,  0, 0 };
+#include "term.h"
 
 static const struct sty *stys[LOWDOWN__MAX] = {
 	NULL, /* LOWDOWN_ROOT */
-	NULL, /* LOWDOWN_BLOCKCODE */
+	&sty_blockcode, /* LOWDOWN_BLOCKCODE */
 	NULL, /* LOWDOWN_BLOCKQUOTE */
 	NULL, /* LOWDOWN_DEFINITION */
 	NULL, /* LOWDOWN_DEFINITION_TITLE */
@@ -109,13 +103,13 @@ static const struct sty *stys[LOWDOWN__MAX] = {
 	&sty_d_emph, /* LOWDOWN_DOUBLE_EMPHASIS */
 	&sty_emph, /* LOWDOWN_EMPHASIS */
 	&sty_highlight, /* LOWDOWN_HIGHLIGHT */
-	&sty_image, /* LOWDOWN_IMAGE */
+	&sty_img, /* LOWDOWN_IMAGE */
 	NULL, /* LOWDOWN_LINEBREAK */
 	&sty_link, /* LOWDOWN_LINK */
 	&sty_t_emph, /* LOWDOWN_TRIPLE_EMPHASIS */
 	&sty_strike, /* LOWDOWN_STRIKETHROUGH */
 	NULL, /* LOWDOWN_SUPERSCRIPT */
-	&sty_foot_ref, /* LOWDOWN_FOOTNOTE_REF */
+	&sty_fref, /* LOWDOWN_FOOTNOTE_REF */
 	NULL, /* LOWDOWN_MATH_BLOCK */
 	&sty_rawhtml, /* LOWDOWN_RAW_HTML */
 	NULL, /* LOWDOWN_ENTITY */
@@ -125,33 +119,6 @@ static const struct sty *stys[LOWDOWN__MAX] = {
 	NULL /* LOWDOWN_DOC_FOOTER */
 };
 
-/* 
- * Special styles.
- * These are invoked in key places, below.
- */
-
-static const struct sty sty_h1 = 	{ 0, 0, 0, 0, 104, 37, 0 };
-static const struct sty sty_hn = 	{ 0, 0, 0, 0,   0, 36, 0 };
-static const struct sty sty_linkalt =	{ 0, 0, 1, 0,   0, 92, 1|2 };
-static const struct sty sty_imgurl = 	{ 0, 0, 0, 1,   0, 32, 2 };
-static const struct sty sty_imgurlbox =	{ 0, 0, 0, 0,   0, 37, 2 };
-static const struct sty sty_foots_div =	{ 0, 0, 0, 0,   0, 37, 0 };
-static const struct sty sty_meta_key =	{ 0, 0, 0, 0,   0, 37, 0 };
-static const struct sty sty_bad_ent = 	{ 0, 0, 0, 0,   0, 37, 0 };
-static const struct sty sty_chng_ins =	{ 0, 0, 0, 0,  47, 30, 0 };
-static const struct sty sty_chng_del =	{ 0, 0, 0, 0, 100,  0, 0 };
-
-/*
- * Prefix styles.
- * These are applied to block-level prefix material.
- */
-
-static const struct sty sty_ddata_pfx =	{ 0, 0, 0, 0,   0, 93, 0 };
-static const struct sty sty_fdef_pfx =	{ 0, 0, 0, 0,   0, 92, 1 };
-static const struct sty sty_bkqt_pfx =	{ 0, 0, 0, 0,   0, 93, 0 };
-static const struct sty sty_oli_pfx =	{ 0, 0, 0, 0,   0, 93, 0 };
-static const struct sty sty_uli_pfx =	{ 0, 0, 0, 0,   0, 93, 0 };
-
 /*
  * Whether the style is not empty (i.e., has style attributes).
  */
@@ -163,13 +130,17 @@ static const struct sty sty_uli_pfx =	{ 0, 0, 0, 0,   0, 93, 0 };
 /* Forward declaration. */
 
 static int
-rndr(struct lowdown_buf *, struct lowdown_metaq *, 
+rndr(struct lowdown_buf *, struct lowdown_metaq *,
 	struct term *, const struct lowdown_node *);
 
 /*
- * Get the column width of a multi-byte sequence.
- * If the sequence is bad, return the number of raw bytes to print.
- * Return <0 on failure (memory), >=0 otherwise.
+ * Get the column width of a multi-byte sequence.  The sequence should
+ * be self-contained, i.e., not straddle multi-byte borders, because the
+ * calculation for UTF-8 columns is local to this function: a split
+ * multi-byte sequence will fail to return the correct number of
+ * printable columns.  If the sequence is bad, return the number of raw
+ * bytes to print.  Return <0 on failure (memory), >=0 otherwise with
+ * the number of printable columns.
  */
 static ssize_t
 rndr_mbswidth(struct term *term, const char *buf, size_t sz)
@@ -177,9 +148,11 @@ rndr_mbswidth(struct term *term, const char *buf, size_t sz)
 	size_t	 	 wsz, csz;
 	const char	*cp;
 	void		*pp;
+	mbstate_t	 mbs;
 
+	memset(&mbs, 0, sizeof(mbstate_t));
 	cp = buf;
-	wsz = mbsnrtowcs(NULL, &cp, sz, 0, NULL);
+	wsz = mbsnrtowcs(NULL, &cp, sz, 0, &mbs);
 	if (wsz == (size_t)-1)
 		return sz;
 
@@ -191,8 +164,9 @@ rndr_mbswidth(struct term *term, const char *buf, size_t sz)
 		term->buf = pp;
 	}
 
+	memset(&mbs, 0, sizeof(mbstate_t));
 	cp = buf;
-	mbsnrtowcs(term->buf, &cp, sz, wsz, NULL);
+	mbsnrtowcs(term->buf, &cp, sz, wsz, &mbs);
 	csz = wcswidth(term->buf, wsz);
 	return csz == (size_t)-1 ? sz : csz;
 }
@@ -239,9 +213,26 @@ rndr_escape(struct term *term, struct lowdown_buf *out,
 }
 
 /*
- * Output style "s" into "out" as an ANSI escape.
- * If "s" does not have any style information, output nothing.
- * Return zero on failure (memory), non-zero on success.
+ * If there's an active style in "s" or s is NULL), then emit an
+ * unstyling escape sequence.  Return zero on failure (memory), non-zero
+ * on success.
+ */
+static int
+rndr_buf_unstyle(const struct term *term,
+	struct lowdown_buf *out, const struct sty *s)
+{
+
+	if (term->opts & LOWDOWN_TERM_NOANSI)
+		return 1;
+	if (s != NULL && !STY_NONEMPTY(s))
+		return 1;
+	return HBUF_PUTSL(out, "\033[0m");
+}
+
+/*
+ * Output style "s" into "out" as an ANSI escape.  If "s" does not have
+ * any style information or is NULL, output nothing.  Return zero on
+ * failure (memory), non-zero on success.
  */
 static int
 rndr_buf_style(const struct term *term,
@@ -249,11 +240,13 @@ rndr_buf_style(const struct term *term,
 {
 	int	has = 0;
 
-	if (!STY_NONEMPTY(s))
+	if (term->opts & LOWDOWN_TERM_NOANSI)
+		return 1;
+	if (s == NULL || !STY_NONEMPTY(s))
 		return 1;
-
 	if (!HBUF_PUTSL(out, "\033["))
 		return 0;
+
 	if (s->bold) {
 		if (!HBUF_PUTSL(out, "1"))
 			return 0;
@@ -277,13 +270,17 @@ rndr_buf_style(const struct term *term,
 		if (!HBUF_PUTSL(out, "9"))
 			return 0;
 	}
-	if (s->bcolour && !(term->opts & LOWDOWN_TERM_NOCOLOUR)) {
+	if (s->bcolour && !(term->opts & LOWDOWN_TERM_NOCOLOUR) &&
+	    ((s->bcolour >= 30 && s->bcolour <= 37) ||
+	     (s->bcolour >= 90 && s->bcolour <= 97))) {
 		if (has++ && !HBUF_PUTSL(out, ";"))
 			return 0;
 		if (!hbuf_printf(out, "%zu", s->bcolour))
 			return 0;
 	}
-	if (s->colour && !(term->opts & LOWDOWN_TERM_NOCOLOUR)) {
+	if (s->colour && !(term->opts & LOWDOWN_TERM_NOCOLOUR) &&
+	    ((s->colour >= 30 && s->colour <= 37) ||
+	     (s->colour >= 90 && s->colour <= 97))) {
 		if (has++ && !HBUF_PUTSL(out, ";"))
 			return 0;
 		if (!hbuf_printf(out, "%zu", s->colour))
@@ -312,7 +309,7 @@ rndr_node_style_apply(struct sty *to, const struct sty *from)
 		to->bold = 0;
 	if (from->under)
 		to->under = 1;
-	else if ((from->override & OSTY_ITALIC))
+	else if ((from->override & OSTY_UNDER))
 		to->under = 0;
 	if (from->bcolour)
 		to->bcolour = from->bcolour;
@@ -339,21 +336,21 @@ rndr_node_style(struct sty *s, const struct lowdown_node *n)
 	switch (n->type) {
 	case LOWDOWN_HEADER:
 		if (n->rndr_header.level > 0)
-			rndr_node_style_apply(s, &sty_hn);
+			rndr_node_style_apply(s, &sty_header_n);
 		else
-			rndr_node_style_apply(s, &sty_h1);
+			rndr_node_style_apply(s, &sty_header_1);
 		break;
 	default:
 		/* FIXME: crawl up nested? */
-		if (n->parent != NULL && 
+		if (n->parent != NULL &&
 		    n->parent->type == LOWDOWN_LINK)
 			rndr_node_style_apply(s, &sty_linkalt);
 		break;
 	}
 
-	if (n->chng == LOWDOWN_CHNG_INSERT) 
+	if (n->chng == LOWDOWN_CHNG_INSERT)
 		rndr_node_style_apply(s, &sty_chng_ins);
-	if (n->chng == LOWDOWN_CHNG_DELETE) 
+	if (n->chng == LOWDOWN_CHNG_DELETE)
 		rndr_node_style_apply(s, &sty_chng_del);
 }
 
@@ -390,18 +387,18 @@ rndr_buf_endstyle(const struct lowdown_node *n)
 
 /*
  * Unsets the current style context given "n" and an optional terminal
- * style "osty", if applies.
- * Return zero on failure (memory), non-zero on success.
+ * style "osty", if applies.  Return zero on failure (memory), non-zero
+ * on success.
  */
 static int
 rndr_buf_endwords(struct term *term, struct lowdown_buf *out,
 	const struct lowdown_node *n, const struct sty *osty)
 {
 
-	if (rndr_buf_endstyle(n) ||
-	    (osty != NULL && STY_NONEMPTY(osty)))
-		return HBUF_PUTSL(out, "\033[0m");
-
+	if (rndr_buf_endstyle(n))
+		return rndr_buf_unstyle(term, out, NULL);
+	if (osty != NULL)
+		return rndr_buf_unstyle(term, out, osty);
 	return 1;
 }
 
@@ -417,7 +414,7 @@ rndr_buf_endline(struct term *term, struct lowdown_buf *out,
 	if (!rndr_buf_endwords(term, out, n, osty))
 		return 0;
 
-	/* 
+	/*
 	 * We can legit be at col == 0 if, for example, we're in a
 	 * literal context with a blank line.
 	 * assert(term->col > 0);
@@ -430,45 +427,52 @@ rndr_buf_endline(struct term *term, struct lowdown_buf *out,
 }
 
 /*
- * Output optional number of newlines before or after content.
- * Return zero on failure, non-zero on success.
+ * Return the printed width of the number up to six digits (we're
+ * probably not going to have more list items than that).
  */
-static int
-rndr_buf_vspace(struct term *term, struct lowdown_buf *out,
-	const struct lowdown_node *n, size_t sz)
+static size_t
+rndr_numlen(size_t sz)
 {
 
-	if (term->last_blank == -1)
-		return 1;
-	while ((size_t)term->last_blank < sz) {
-		if (!HBUF_PUTSL(out, "\n"))
-			return 0;
-		term->last_blank++;
-	}
-	term->col = 0;
+	if (sz > 100000)
+		return 6;
+	if (sz > 10000)
+		return 5;
+	if (sz > 1000)
+		return 4;
+	if (sz > 100)
+		return 3;
+	if (sz > 10)
+		return 2;
 	return 1;
 }
 
 /*
  * Output prefixes of the given node in the style further accumulated
- * from the parent nodes.
+ * from the parent nodes.  "Depth" is set to how deep we are, starting
+ * at -1 (the root).
  * Return zero on failure (memory), non-zero on success.
  */
 static int
 rndr_buf_startline_prefixes(struct term *term,
 	struct sty *s, const struct lowdown_node *n,
-	struct lowdown_buf *out)
+	struct lowdown_buf *out, size_t *depth)
 {
 	struct sty			 sinner;
-	const struct lowdown_node	*np;
-	size_t	 			 i, emit;
+	const struct pfx		*pfx;
+	size_t	 			 i, emit, len;
 	int	 		 	 pstyle = 0;
 	enum hlist_fl			 fl;
 
 	if (n->parent != NULL &&
-	    !rndr_buf_startline_prefixes(term, s, n->parent, out))
+	    !rndr_buf_startline_prefixes(term, s, n->parent, out, depth))
 		return 0;
 
+	if (n->parent == NULL) {
+		assert(n->type == LOWDOWN_ROOT);
+		*depth = -1;
+	}
+
 	/*
 	 * The "sinner" value is temporary for only this function.
 	 * This allows us to set a temporary style mask that only
@@ -494,6 +498,21 @@ rndr_buf_startline_prefixes(struct term *term,
 	assert(i <= term->stackpos);
 	emit = term->stack[i].lines++;
 
+	/*
+	 * If we're below the document root and not a header, that means
+	 * we're in a body part.  Emit the general body indentation.
+	 */
+
+	if (*depth == 0 && n->type != LOWDOWN_HEADER) {
+		if (!hbuf_puts(out, pfx_body.text))
+			return 0;
+		rndr_buf_advance(term, pfx_body.cols);
+	} else if (*depth == 0) {
+		if (!hbuf_puts(out, pfx_header.text))
+			return 0;
+		rndr_buf_advance(term, pfx_header.cols);
+	}
+
 	/*
 	 * Output any prefixes.
 	 * Any output must have rndr_buf_style() and set pstyle so that
@@ -501,32 +520,14 @@ rndr_buf_startline_prefixes(struct term *term,
 	 */
 
 	switch (n->type) {
-	case LOWDOWN_TABLE_BLOCK:
-	case LOWDOWN_PARAGRAPH:
-	case LOWDOWN_DEFINITION_TITLE:
-		/*
-		 * Collapse leading white-space if we're already within
-		 * a margin-bearing block statement.
-		 */
-
-		for (np = n->parent; np != NULL; np = np->parent)
-			if (np->type == LOWDOWN_LISTITEM ||
-			    np->type == LOWDOWN_BLOCKQUOTE ||
-			    np->type == LOWDOWN_FOOTNOTE_DEF)
-				break;
-		if (np == NULL) {
-			if (!HBUF_PUTSL(out, "    "))
-				return 0;
-			rndr_buf_advance(term, 4);
-		}
-		break;
 	case LOWDOWN_BLOCKCODE:
+		rndr_node_style_apply(&sinner, &sty_bkcd_pfx);
 		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
-		if (!HBUF_PUTSL(out, "      "))
+		if (!hbuf_puts(out, pfx_bkcd.text))
 			return 0;
-		rndr_buf_advance(term, 6);
+		rndr_buf_advance(term, pfx_bkcd.cols);
 		break;
 	case LOWDOWN_ROOT:
 		if (!rndr_buf_style(term, out, &sinner))
@@ -541,47 +542,64 @@ rndr_buf_startline_prefixes(struct term *term,
 		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
-		if (!HBUF_PUTSL(out, "    | "))
+		if (!hbuf_puts(out, pfx_bkqt.text))
 			return 0;
-		rndr_buf_advance(term, 6);
+		rndr_buf_advance(term, pfx_bkqt.cols);
 		break;
 	case LOWDOWN_DEFINITION_DATA:
-		rndr_node_style_apply(&sinner, &sty_ddata_pfx);
+		rndr_node_style_apply(&sinner, &sty_dli_pfx);
 		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
-		if (emit == 0 && !HBUF_PUTSL(out, "    : "))
-			return 0;
-		else if (emit != 0 && !HBUF_PUTSL(out, "      "))
-			return 0;
-		rndr_buf_advance(term, 6);
+		if (emit == 0) {
+			if (!hbuf_puts(out, pfx_dli_1.text))
+				return 0;
+			rndr_buf_advance(term, pfx_dli_1.cols);
+		} else {
+			if (!hbuf_puts(out, pfx_dli_n.text))
+				return 0;
+			rndr_buf_advance(term, pfx_dli_n.cols);
+		}
 		break;
 	case LOWDOWN_FOOTNOTE_DEF:
 		rndr_node_style_apply(&sinner, &sty_fdef_pfx);
 		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
-		if (emit == 0 && !hbuf_printf
-		    (out, "%2zu. ", n->rndr_footnote_def.num))
-			return 0;
-		else if (emit != 0 && !HBUF_PUTSL(out, "    "))
-			return 0;
-		rndr_buf_advance(term, 4);
+		if (emit == 0) {
+			if (!hbuf_printf(out, "%2zu. ",
+			     n->rndr_footnote_def.num))
+				return 0;
+			len = rndr_numlen(n->rndr_footnote_def.num);
+			if (len + 2 > pfx_fdef_1.cols)
+				len += 2;
+			else
+				len = pfx_fdef_1.cols;
+			rndr_buf_advance(term, len);
+		} else {
+			if (!hbuf_puts(out, pfx_fdef_n.text))
+				return 0;
+			rndr_buf_advance(term, pfx_fdef_n.cols);
+		}
 		break;
 	case LOWDOWN_HEADER:
-		/* Use the same colour as the text following. */
-
 		if (n->rndr_header.level == 0)
-			break;
+			pfx = &pfx_header_1;
+		else
+			pfx = &pfx_header_n;
 		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
-		for (i = 0; i < n->rndr_header.level + 1; i++)
-			if (!HBUF_PUTSL(out, "#"))
+		for (i = 0; i < n->rndr_header.level + 1; i++) {
+			if (!hbuf_puts(out, pfx->text))
 				return 0;
-		if (!HBUF_PUTSL(out, " "))
-			return 0;
-		rndr_buf_advance(term, i + 1);
+			rndr_buf_advance(term, pfx->cols);
+		}
+		if (pfx->cols) {
+			if (!HBUF_PUTSL(out, " "))
+				return 0;
+			rndr_buf_advance(term, 1);
+		}
 		break;
 	case LOWDOWN_LISTITEM:
 		if (n->parent == NULL ||
@@ -591,45 +609,54 @@ rndr_buf_startline_prefixes(struct term *term,
 		/* Don't print list item prefix after first. */
 
 		if (emit) {
-			if (!HBUF_PUTSL(out, "      "))
+			if (!hbuf_puts(out, pfx_li_n.text))
 				return 0;
-			rndr_buf_advance(term, 6);
+			rndr_buf_advance(term, pfx_li_n.cols);
 			break;
 		}
 
 		/* List item prefix depends upon type. */
 
 		fl = n->rndr_list.flags;
-		rndr_node_style_apply(&sinner,
-			(fl & HLIST_FL_ORDERED) ?
-			&sty_oli_pfx : &sty_uli_pfx);
+		rndr_node_style_apply(&sinner, &sty_li_pfx);
 		if (!rndr_buf_style(term, out, &sinner))
 			return 0;
 		pstyle = 1;
 
-		if (fl & HLIST_FL_CHECKED) {
-			if (!hbuf_puts(out, "    ☑ "))
-				return 0;
-		} else if (fl & HLIST_FL_UNCHECKED) {
-			if (!hbuf_puts(out, "    ☐ "))
-				return 0;
-		} else if (fl & HLIST_FL_UNORDERED) {
-			if (!hbuf_puts(out, "    · "))
+		if (fl & HLIST_FL_CHECKED)
+			pfx = &pfx_uli_c1;
+		else if (fl & HLIST_FL_UNCHECKED)
+			pfx = &pfx_uli_nc1;
+		else if (fl & HLIST_FL_UNORDERED)
+			pfx = &pfx_uli_1;
+		else
+			pfx = &pfx_oli_1;
+
+		if (pfx == &pfx_oli_1) {
+			if (!hbuf_printf(out, "%2zu. ",
+			     n->rndr_listitem.num))
 				return 0;
+			len = rndr_numlen(n->rndr_listitem.num);
+			if (len + 2 > pfx->cols)
+				len += 2;
+			else
+				len = pfx->cols;
 		} else {
-			if (!hbuf_printf(out, "%4zu. ", n->rndr_listitem.num))
+			if (pfx->text != NULL &&
+			    !hbuf_puts(out, pfx->text))
 				return 0;
+			len = pfx->cols;
 		}
-		rndr_buf_advance(term, 6);
+		rndr_buf_advance(term, len);
 		break;
 	default:
 		break;
 	}
 
-	if (pstyle && STY_NONEMPTY(&sinner))
-		if (!HBUF_PUTSL(out, "\033[0m"))
-			return 0;
+	if (pstyle && !rndr_buf_unstyle(term, out, &sinner))
+		return 0;
 
+	(*depth)++;
 	return 1;
 }
 
@@ -639,22 +666,58 @@ rndr_buf_startline_prefixes(struct term *term,
  * Return zero on failure (memory), non-zero on success.
  */
 static int
-rndr_buf_startline(struct term *term, struct lowdown_buf *out, 
+rndr_buf_startline(struct term *term, struct lowdown_buf *out,
 	const struct lowdown_node *n, const struct sty *osty)
 {
 	struct sty	 s;
+	size_t		 depth = 0;
 
 	assert(term->last_blank);
 	assert(term->col == 0);
 
 	memset(&s, 0, sizeof(struct sty));
-	if (!rndr_buf_startline_prefixes(term, &s, n, out))
+	if (!rndr_buf_startline_prefixes(term, &s, n, out, &depth))
 		return 0;
 	if (osty != NULL)
 		rndr_node_style_apply(&s, osty);
 	return rndr_buf_style(term, out, &s);
 }
 
+/*
+ * Output optional number of newlines before or after content.
+ * Return zero on failure, non-zero on success.
+ */
+static int
+rndr_buf_vspace(struct term *term, struct lowdown_buf *out,
+	const struct lowdown_node *n, size_t sz)
+{
+	const struct lowdown_node	*prev;
+
+	if (term->last_blank == -1)
+		return 1;
+
+	prev = n->parent == NULL ? NULL :
+		TAILQ_PREV(n, lowdown_nodeq, entries);
+
+	assert(sz > 0);
+	while ((size_t)term->last_blank < sz) {
+		if (term->col || prev == NULL) {
+			if (!HBUF_PUTSL(out, "\n"))
+				return 0;
+		} else {
+			if (!rndr_buf_startline
+			    (term, out, n->parent, NULL))
+				return 0;
+			if (!rndr_buf_endline
+			    (term, out, n->parent, NULL))
+				return 0;
+		}
+		term->last_blank++;
+		term->col = 0;
+	}
+	return 1;
+}
+
 /*
  * Ascend to the root of the parse tree from rndr_buf_startwords(),
  * accumulating styles as we do so.
@@ -694,7 +757,7 @@ rndr_buf_startwords(struct term *term, struct lowdown_buf *out,
  * Return zero on failure, non-zero on success.
  */
 static int
-rndr_buf_literal(struct term *term, struct lowdown_buf *out, 
+rndr_buf_literal(struct term *term, struct lowdown_buf *out,
 	const struct lowdown_node *n, const struct lowdown_buf *in,
 	const struct sty *osty)
 {
@@ -710,7 +773,7 @@ rndr_buf_literal(struct term *term, struct lowdown_buf *out,
 		if (!rndr_buf_startline(term, out, n, osty))
 			return 0;
 
-		/* 
+		/*
 		 * No need to record the column width here because we're
 		 * going to reset to zero anyway.
 		 */
@@ -731,7 +794,7 @@ rndr_buf_literal(struct term *term, struct lowdown_buf *out,
  * Return zero on failure, non-zero on success.
  */
 static int
-rndr_buf(struct term *term, struct lowdown_buf *out, 
+rndr_buf(struct term *term, struct lowdown_buf *out,
 	const struct lowdown_node *n, const struct lowdown_buf *in,
 	const struct sty *osty)
 {
@@ -751,7 +814,7 @@ rndr_buf(struct term *term, struct lowdown_buf *out,
 	while (i < in->size) {
 		needspace = isspace((unsigned char)in->data[i]);
 
-		while (i < in->size && 
+		while (i < in->size &&
 		       isspace((unsigned char)in->data[i]))
 			i++;
 
@@ -763,7 +826,7 @@ rndr_buf(struct term *term, struct lowdown_buf *out,
 			i++;
 		len = &in->data[i] - start;
 
-		/* 
+		/*
 		 * If we cross our maximum width and are preceded by a
 		 * space, then break.
 		 * (Leaving out the check for a space will cause
@@ -772,7 +835,7 @@ rndr_buf(struct term *term, struct lowdown_buf *out,
 		 * This will also unset the current style.
 		 */
 
-		if ((needspace || 
+		if ((needspace ||
 	 	     (out->size && isspace
 		      ((unsigned char)out->data[out->size - 1]))) &&
 		    term->col && term->col + len > term->maxcol) {
@@ -845,7 +908,7 @@ rndr_entity(struct lowdown_buf *buf, int32_t val)
 		return hbuf_putc(buf, 192 + val / 64) &&
 			hbuf_putc(buf, 128 + val % 64);
 
-       	if (val - 0xd800u < 0x800) 
+	if (val - 0xd800u < 0x800)
 		return 1;
 
        	if (val < 0x10000)
@@ -893,7 +956,7 @@ rndr_table(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 {
 	size_t				*widths = NULL;
 	const struct lowdown_node	*row, *top, *cell;
-	struct lowdown_buf		*celltmp = NULL, 
+	struct lowdown_buf		*celltmp = NULL,
 					*rowtmp = NULL;
 	size_t				 col, i, j, maxcol, sz;
 	ssize_t			 	 last_blank;
@@ -924,7 +987,7 @@ rndr_table(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 				assert(i < n->rndr_table.columns);
 				hbuf_truncate(celltmp);
 
-				/* 
+				/*
 				 * Simulate that we're starting within
 				 * the line by unsetting last_blank,
 				 * having a non-zero column, and an
@@ -935,6 +998,7 @@ rndr_table(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 				maxcol = p->maxcol;
 				last_blank = p->last_blank;
 				col = p->col;
+
 				p->last_blank = 0;
 				p->maxcol = SIZE_MAX;
 				p->col = 1;
@@ -961,6 +1025,7 @@ rndr_table(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 				maxcol = p->maxcol;
 				last_blank = p->last_blank;
 				col = p->col;
+
 				p->last_blank = 0;
 				p->maxcol = SIZE_MAX;
 				p->col = 1;
@@ -969,7 +1034,7 @@ rndr_table(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 				assert(widths[i] >= p->col);
 				sz = widths[i] - p->col;
 
-				/* 
+				/*
 				 * Alignment is either beginning,
 				 * ending, or splitting the remaining
 				 * spaces around the word.
@@ -977,7 +1042,7 @@ rndr_table(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 				 * the case of centre.
 				 */
 
-				flags = cell->rndr_table_cell.flags & 
+				flags = cell->rndr_table_cell.flags &
 					HTBL_FL_ALIGNMASK;
 				if (flags == HTBL_FL_ALIGN_RIGHT)
 					for (j = 0; j < sz; j++)
@@ -995,7 +1060,7 @@ rndr_table(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 						if (!HBUF_PUTSL(rowtmp, " "))
 							goto out;
 				if (flags == HTBL_FL_ALIGN_CENTER) {
-					sz = (sz % 2) ? 
+					sz = (sz % 2) ?
 						(sz / 2) + 1 : (sz / 2);
 					for (j = 0; j < sz; j++)
 						if (!HBUF_PUTSL(rowtmp, " "))
@@ -1005,12 +1070,17 @@ rndr_table(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 				p->last_blank = last_blank;
 				p->col = col;
 				p->maxcol = maxcol;
-				if (TAILQ_NEXT(cell, entries) != NULL &&
-				    !HBUF_PUTSL(rowtmp, " | "))
+
+				if (TAILQ_NEXT(cell, entries) == NULL)
+					continue;
+
+				if (!rndr_buf_style(p, rowtmp, &sty_table) ||
+				    !hbuf_printf(rowtmp, " %s ", ifx_table_col) ||
+				    !rndr_buf_unstyle(p, rowtmp, &sty_table))
 					goto out;
 			}
 
-			/* 
+			/*
 			 * Some magic here.
 			 * First, emulate rndr() by setting the
 			 * stackpos to the table, which is required for
@@ -1039,18 +1109,19 @@ rndr_table(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 			p->stackpos++;
 			if (!rndr_stackpos_init(p, n))
 				goto out;
-			if (!rndr_buf_startline(p, ob, n, NULL))
+			if (!rndr_buf_startline(p, ob, n, &sty_table))
 				goto out;
 			for (i = 0; i < n->rndr_table.columns; i++) {
 				for (j = 0; j < widths[i]; j++)
-					if (!HBUF_PUTSL(ob, "-"))
+					if (!hbuf_puts(ob, ifx_table_row))
 						goto out;
 				if (i < n->rndr_table.columns - 1 &&
-				    !HBUF_PUTSL(ob, "|-"))
+				    !hbuf_printf(ob, "%s%s",
+				    ifx_table_col, ifx_table_row))
 					goto out;
 			}
 			rndr_buf_advance(p, 1);
-			if (!rndr_buf_endline(p, ob, n, NULL))
+			if (!rndr_buf_endline(p, ob, n, &sty_table))
 				goto out;
 			if (!rndr_buf_vspace(p, ob, n, 1))
 				goto out;
@@ -1070,30 +1141,30 @@ static int
 rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 	struct term *p, const struct lowdown_node *n)
 {
-	const struct lowdown_node	*child, *prev;
+	const struct lowdown_node	*child, *nn;
 	struct lowdown_meta		*m;
 	struct lowdown_buf		*metatmp;
 	int32_t				 entity;
-	size_t				 i, col;
+	size_t				 i, col, vs;
 	ssize_t			 	 last_blank;
 	int				 rc;
-	
+
 	/* Current nodes we're servicing. */
 
 	if (!rndr_stackpos_init(p, n))
 		return 0;
 
-	prev = n->parent == NULL ? NULL :
-		TAILQ_PREV(n, lowdown_nodeq, entries);
+	/*
+	 * Vertical space before content.  Vertical space (>1 space) is
+	 * suppressed for normal blocks when in a non-block list, as the
+	 * list item handles any spacing.  Furthermore, definition list
+	 * data also has its spaces suppressed because this is relegated
+	 * to the title.  The root gets the vertical margin as well.
+	 */
 
-	/* Vertical space before content. */
-
-	rc = 1;
+	vs = 0;
 	switch (n->type) {
 	case LOWDOWN_ROOT:
-
-		/* Emit vmargin. */
-
 		for (i = 0; i < p->vmargin; i++)
 			if (!HBUF_PUTSL(ob, "\n"))
 				return 0;
@@ -1103,56 +1174,45 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 	case LOWDOWN_BLOCKHTML:
 	case LOWDOWN_BLOCKQUOTE:
 	case LOWDOWN_DEFINITION:
+	case LOWDOWN_DEFINITION_TITLE:
 	case LOWDOWN_FOOTNOTES_BLOCK:
 	case LOWDOWN_FOOTNOTE_DEF:
 	case LOWDOWN_HEADER:
 	case LOWDOWN_LIST:
 	case LOWDOWN_TABLE_BLOCK:
 	case LOWDOWN_PARAGRAPH:
-		/*
-		 * Blocks in a definition list get special treatment
-		 * because we only put one newline between the title and
-		 * the data regardless of its contents.
-		 */
-
-		if (n->parent != NULL && 
-		    n->parent->type == LOWDOWN_LISTITEM &&
-		    n->parent->parent != NULL &&
-		    n->parent->parent->type == 
-		      LOWDOWN_DEFINITION_DATA &&
-		    prev == NULL)
-			rc = rndr_buf_vspace(p, ob, n, 1);
-		else
-			rc = rndr_buf_vspace(p, ob, n, 2);
+		vs = 2;
+		for (nn = n->parent; nn != NULL; nn = nn->parent) {
+			if (nn->type != LOWDOWN_LISTITEM)
+				continue;
+			vs = (nn->rndr_listitem.flags & HLIST_FL_BLOCK) ? 2 : 1;
+			break;
+		}
 		break;
 	case LOWDOWN_MATH_BLOCK:
-		if (n->rndr_math.blockmode)
-			rc = rndr_buf_vspace(p, ob, n, 1);
+		vs = n->rndr_math.blockmode ? 1 : 0;
 		break;
 	case LOWDOWN_DEFINITION_DATA:
-		/* Vertical space if previous block-mode data. */
-
-		if (n->parent != NULL &&
-		    n->parent->type == LOWDOWN_DEFINITION &&
-		    (n->parent->rndr_definition.flags &
-		     HLIST_FL_BLOCK) &&
-		    prev != NULL &&
-		    prev->type == LOWDOWN_DEFINITION_DATA)
-			rc = rndr_buf_vspace(p, ob, n, 2);
-		else
-			rc = rndr_buf_vspace(p, ob, n, 1);
-		break;
-	case LOWDOWN_DEFINITION_TITLE:
 	case LOWDOWN_HRULE:
 	case LOWDOWN_LINEBREAK:
-	case LOWDOWN_LISTITEM:
 	case LOWDOWN_META:
-		rc = rndr_buf_vspace(p, ob, n, 1);
+		vs = 1;
+		break;
+	case LOWDOWN_LISTITEM:
+		vs = 1;
+		if (n->rndr_listitem.flags & HLIST_FL_BLOCK) {
+			for (nn = n->parent; nn != NULL; nn = nn->parent)
+				if (nn->type == LOWDOWN_LISTITEM ||
+				    nn->type == LOWDOWN_DEFINITION_DATA)
+					break;
+			vs = nn == NULL ? 2 : 1;
+		}
 		break;
 	default:
 		break;
 	}
-	if (!rc)
+
+	if (vs > 0 && !rndr_buf_vspace(p, ob, n, vs))
 		return 0;
 
 	/* Output leading content. */
@@ -1160,26 +1220,23 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 	switch (n->type) {
 	case LOWDOWN_FOOTNOTES_BLOCK:
 		hbuf_truncate(p->tmp);
-		if (!HBUF_PUTSL(p->tmp, "~~~~~~~~"))
-			return 0;
-		if (!rndr_buf(p, ob, n, p->tmp, &sty_foots_div))
+		if (!hbuf_puts(p->tmp, ifx_foot) ||
+		    !rndr_buf(p, ob, n, p->tmp, &sty_foot))
 			return 0;
 		break;
 	case LOWDOWN_SUPERSCRIPT:
 		hbuf_truncate(p->tmp);
-		if (!HBUF_PUTSL(p->tmp, "^"))
-			return 0;
-		if (!rndr_buf(p, ob, n, p->tmp, NULL))
+		if (!hbuf_puts(p->tmp, ifx_super) ||
+		    !rndr_buf(p, ob, n, p->tmp, NULL))
 			return 0;
 		break;
 	case LOWDOWN_META:
-		if (!rndr_buf(p, ob, n, 
+		if (!rndr_buf(p, ob, n,
 		    &n->rndr_meta.key, &sty_meta_key))
 			return 0;
 		hbuf_truncate(p->tmp);
-		if (!HBUF_PUTSL(p->tmp, ": "))
-			return 0;
-		if (!rndr_buf(p, ob, n, p->tmp, &sty_meta_key))
+		if (!hbuf_puts(p->tmp, ifx_meta_key) ||
+		    !rndr_buf(p, ob, n, p->tmp, &sty_meta_key))
 			return 0;
 		if (mq == NULL)
 			break;
@@ -1238,17 +1295,18 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 
 	/* Output content. */
 
+	rc = 1;
 	switch (n->type) {
 	case LOWDOWN_HRULE:
 		hbuf_truncate(p->tmp);
-		if (!HBUF_PUTSL(p->tmp, "~~~~~~~~"))
+		if (!hbuf_puts(p->tmp, ifx_hrule))
 			return 0;
 		rc = rndr_buf(p, ob, n, p->tmp, NULL);
 		break;
 	case LOWDOWN_FOOTNOTE_REF:
 		hbuf_truncate(p->tmp);
-		if (!hbuf_printf(p->tmp, "[%zu]", 
-		    n->rndr_footnote_ref.num))
+		if (!hbuf_printf(p->tmp, "%s%zu%s", ifx_fref_left,
+		    n->rndr_footnote_ref.num, ifx_fref_right))
 			return 0;
 		rc = rndr_buf(p, ob, n, p->tmp, NULL);
 		break;
@@ -1285,7 +1343,7 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 			    (p->tmp, &n->rndr_autolink.link))
 				return 0;
 			rc = rndr_buf(p, ob, n, p->tmp, NULL);
-		} else 
+		} else
 			rc = rndr_buf(p, ob, n, &n->rndr_autolink.link, NULL);
 		break;
 	case LOWDOWN_LINK:
@@ -1302,7 +1360,7 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 			    (p->tmp, &n->rndr_link.link))
 				return 0;
 			rc = rndr_buf(p, ob, n, p->tmp, NULL);
-		} else 
+		} else
 			rc = rndr_buf(p, ob, n, &n->rndr_link.link, NULL);
 		break;
 	case LOWDOWN_IMAGE:
@@ -1317,15 +1375,16 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 		}
 		if (p->opts & LOWDOWN_TERM_NOLINK) {
 			hbuf_truncate(p->tmp);
-			if (!HBUF_PUTSL(p->tmp, "[Image]"))
+			if (!hbuf_puts(p->tmp, ifx_imgbox_left) ||
+			    !hbuf_puts(p->tmp, ifx_imgbox_right))
 				return 0;
-			rc = rndr_buf(p, ob, n, p->tmp, &sty_imgurlbox);
+			rc = rndr_buf(p, ob, n, p->tmp, &sty_imgbox);
 			break;
 		}
 		hbuf_truncate(p->tmp);
-		if (!HBUF_PUTSL(p->tmp, "[Image: "))
-			return 0;
-		if (!rndr_buf(p, ob, n, p->tmp, &sty_imgurlbox))
+		if (!hbuf_puts(p->tmp, ifx_imgbox_left) ||
+		    !hbuf_puts(p->tmp, ifx_imgbox_sep) ||
+		    !rndr_buf(p, ob, n, p->tmp, &sty_imgbox))
 			return 0;
 		if (p->opts & LOWDOWN_TERM_SHORTLINK) {
 			hbuf_truncate(p->tmp);
@@ -1335,13 +1394,13 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 			if (!rndr_buf(p, ob, n, p->tmp, &sty_imgurl))
 				return 0;
 		} else
-			if (!rndr_buf(p, ob, n, 
+			if (!rndr_buf(p, ob, n,
 			    &n->rndr_image.link, &sty_imgurl))
 				return 0;
 		hbuf_truncate(p->tmp);
-		if (!HBUF_PUTSL(p->tmp, "]"))
+		if (!hbuf_puts(p->tmp, ifx_imgbox_right))
 			return 0;
-		rc = rndr_buf(p, ob, n, p->tmp, &sty_imgurlbox);
+		rc = rndr_buf(p, ob, n, p->tmp, &sty_imgbox);
 		break;
 	case LOWDOWN_NORMAL_TEXT:
 		rc = rndr_buf(p, ob, n, &n->rndr_normal_text.text, NULL);
@@ -1354,36 +1413,7 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 
 	/* Trailing block spaces. */
 
-	rc = 1;
-	switch (n->type) {
-	case LOWDOWN_BLOCKCODE:
-	case LOWDOWN_BLOCKHTML:
-	case LOWDOWN_BLOCKQUOTE:
-	case LOWDOWN_DEFINITION:
-	case LOWDOWN_FOOTNOTES_BLOCK:
-	case LOWDOWN_FOOTNOTE_DEF:
-	case LOWDOWN_HEADER:
-	case LOWDOWN_LIST:
-	case LOWDOWN_PARAGRAPH:
-	case LOWDOWN_TABLE_BLOCK:
-		rc = rndr_buf_vspace(p, ob, n, 2);
-		break;
-	case LOWDOWN_MATH_BLOCK:
-		if (n->rndr_math.blockmode)
-			rc = rndr_buf_vspace(p, ob, n, 1);
-		break;
-	case LOWDOWN_DOC_HEADER:
-		if (!TAILQ_EMPTY(&n->children))
-			rc = rndr_buf_vspace(p, ob, n, 2);
-		break;
-	case LOWDOWN_DEFINITION_DATA:
-	case LOWDOWN_DEFINITION_TITLE:
-	case LOWDOWN_HRULE:
-	case LOWDOWN_LISTITEM:
-	case LOWDOWN_META:
-		rc = rndr_buf_vspace(p, ob, n, 1);
-		break;
-	case LOWDOWN_ROOT:
+	if (n->type == LOWDOWN_ROOT) {
 		if (!rndr_buf_vspace(p, ob, n, 1))
 			return 0;
 		while (ob->size && ob->data[ob->size - 1] == '\n')
@@ -1396,12 +1426,9 @@ rndr(struct lowdown_buf *ob, struct lowdown_metaq *mq,
 		for (i = 0; i < p->vmargin; i++)
 			if (!HBUF_PUTSL(ob, "\n"))
 				return 0;
-		break;
-	default:
-		break;
 	}
 
-	return rc;
+	return 1;
 }
 
 int
