@@ -1,4 +1,4 @@
-/*	$Id: tree.c,v 1.34 2021/02/17 13:31:06 kristaps Exp $ */
+/*	$Id$ */
 /*
  * Copyright (c) 2017--2021 Kristaps Dzonsons <kristaps@bsd.lv>
  *
@@ -300,9 +300,18 @@ rndr(struct lowdown_buf *ob,
 		if (!rndr_indent(ob, indent + 1))
 			return 0;
 		if (!hbuf_printf(ob, "scope: %s\n",
-		    HLIST_FL_BLOCK & root->rndr_listitem.flags ? 
+		    (root->rndr_listitem.flags & HLIST_FL_BLOCK) ?
 		    "block" : "span"))
 			return 0;
+		if (!(root->rndr_listitem.flags &
+		     (HLIST_FL_CHECKED | HLIST_FL_UNCHECKED)))
+			break;
+		if (!rndr_indent(ob, indent + 1))
+			return 0;
+		if (!hbuf_printf(ob, "check status: %s\n",
+		    (root->rndr_listitem.flags & HLIST_FL_CHECKED) ?
+		    "checked" : "unchecked"))
+			return 0;
 		break;
 	case LOWDOWN_LIST:
 		if (!rndr_indent(ob, indent + 1))
@@ -349,6 +358,18 @@ rndr(struct lowdown_buf *ob,
 		if (!HBUF_PUTSL(ob, "\n"))
 			return 0;
 		break;
+	case LOWDOWN_LINK_AUTO:
+		if (root->rndr_autolink.link.size) {
+			if (!rndr_indent(ob, indent + 1))
+				return 0;
+			if (!HBUF_PUTSL(ob, "link: "))
+				return 0;
+			if (!rndr_short(ob, &root->rndr_autolink.link))
+				return 0;
+			if (!HBUF_PUTSL(ob, "\n"))
+				return 0;
+		}
+		break;
 	case LOWDOWN_LINK:
 		if (root->rndr_link.title.size) {
 			if (!rndr_indent(ob, indent + 1))
@@ -360,6 +381,16 @@ rndr(struct lowdown_buf *ob,
 			if (!HBUF_PUTSL(ob, "\n"))
 				return 0;
 		}
+		if (root->rndr_link.link.size) {
+			if (!rndr_indent(ob, indent + 1))
+				return 0;
+			if (!HBUF_PUTSL(ob, "link: "))
+				return 0;
+			if (!rndr_short(ob, &root->rndr_link.link))
+				return 0;
+			if (!HBUF_PUTSL(ob, "\n"))
+				return 0;
+		}
 		break;
 	case LOWDOWN_NORMAL_TEXT:
 		if (!rndr_indent(ob, indent + 1))
